function mle = lyapunov(dydt, y0, t_trans, d0, delta_t, t_max)
% Function to calculate the maximal Lyapunov exponent of
% an attractor of a system dy/dt=f(y, t) (it is implicitly 
% assumed that f(y, t) is independent of t)
% Inputs: dydt - handle to a function that calculates dy/dt
%         y0 - initial condition in the basin of attraction
%         t_trans - time for transients to disappear
%         d0 - initial separation of two orbits
%         delta_t - time step
%         t_max - length of time (after t_trans) to integrate for
%                 (steps 3-5 are repeated until this time is reached)
% Outputs: mle - running average of the maximal Lyapunov
%                exponent at each time step

% integrate to get rid of transient behaviour:
[t, Y] = ode45(dydt, [0, t_trans], y0);
y1 = Y(size(Y, 1), :)';    % final value of solution is y1
y2 = y1+[d0; zeros(length(y1)-1, 1)];   % perturb by d0 to get y2
N_steps = t_max/delta_t;   % number of steps required
sl = zeros(1, N_steps);
sum = 0;
for I = 1:N_steps
   % integrate both orbits by a time delta_t:
   y1 = rk4step(dydt, [0, delta_t], y1);
   y2 = rk4step(dydt, [0, delta_t], y2);
   d1 = norm(y2-y1);              % new separation
   lambda = log(d1/d0)/delta_t;   % Lyapunov exponent
   sum = sum+lambda;        % running sum of Lyapunov exponents
   sl(I) = sum;
   y2 = y1+(y2-y1)*d0/d1;   % renormalise y2 so separation is d0
end
% divide running sum by number of iterations to get running average:
mle = sl(1:I)./(1:I);
